import { IgcHTMLElement } from "igniteui-webcomponents-core";
import { delegateCombine, delegateRemove } from "igniteui-webcomponents-core";
import { IgcToolActionCollection } from "./igc-tool-action-collection";
import { Visibility_$type } from "igniteui-webcomponents-core";
import { IgcToolCommandEventArgs } from "./igc-tool-command-event-args";
import { IgcToolActionPerformedEventArgs } from "./igc-tool-action-performed-event-args";
import { ToolAction } from "./ToolAction";
import { TypeRegistrar } from "igniteui-webcomponents-core";
import { NamePatcher, getAllPropertyNames, fromSpinal, toSpinal, brushToString, stringToBrush, ensureBool, toBrushCollection, fromBrushCollection, brushCollectionToString, ensureEnum, enumToString, initializePropertiesFromCss } from "igniteui-webcomponents-core";
import { ToolActionCollection as ToolActionCollection_internal } from "./ToolActionCollection";
import { SyncableObservableCollection$1 } from "igniteui-webcomponents-core";
import { FontInfo } from "igniteui-webcomponents-core";
/**
 * Represents base class for all tool actions in toolbar
*/
export let IgcToolActionComponent = /*@__PURE__*/ (() => {
    class IgcToolActionComponent extends IgcHTMLElement {
        createImplementation() {
            return null;
        }
        /**
         * @hidden
         */
        get i() {
            return this._implementation;
        } /**
                                     * @hidden
                                     */
        static _createFromInternal(internal) {
            if (!internal) {
                return null;
            }
            if (!internal.$type) {
                return null;
            }
            let name = internal.$type.name;
            let externalName = "Igc" + name + "Component";
            if (!TypeRegistrar.isRegistered(externalName)) {
                return null;
            }
            return TypeRegistrar.create(externalName);
        }
        onImplementationCreated() {
        }
        constructor() {
            super();
            this._settingAttributes = false;
            this._attached = false;
            this._queuedSetAttributes = [];
            this._updatingFromAttribute = false;
            this._actions = null;
            this.__p = null;
            this._hasUserValues = new Set();
            this._stylingContainer = null;
            this._stylingParent = null;
            this._inStyling = false;
            this._onCommand = null;
            this._onCommand_wrapped = null;
            this._performed = null;
            this._performed_wrapped = null;
            if (this._styling) {
                NamePatcher.ensureStylablePatched(Object.getPrototypeOf(this));
            }
            this._implementation = this.createImplementation();
            this._implementation.externalObject = this;
            this.onImplementationCreated();
            if (this._initializeAdapters) {
                this._initializeAdapters();
            }
        }
        _enqueueSetAttribute(attrName, attrValue) {
            this._queuedSetAttributes.push({ attrName: attrName, attrValue: attrValue });
        }
        _flushQueuedAttributes() {
            this._settingAttributes = true;
            for (let i = 0; i < this._queuedSetAttributes.length; i++) {
                this.setAttribute(this._queuedSetAttributes[i].attrName, this._queuedSetAttributes[i].attrValue);
            }
            this._settingAttributes = false;
            this._queuedSetAttributes.length = 0;
        }
        _a(attrName, attrValue) {
            if (this._updatingFromAttribute) {
                return;
            }
            if (attrValue) {
                attrValue = attrValue.toString();
            }
            this._settingAttributes = true;
            attrName = toSpinal(attrName);
            if (this._attached) {
                this.setAttribute(attrName, attrValue);
            }
            else {
                this._enqueueSetAttribute(attrName, attrValue);
            }
            this._settingAttributes = false;
        }
        connectedCallback() {
            if (super["connectedCallback"]) {
                super["connectedCallback"]();
            }
            if (this.i.connectedCallback) {
                this.i.connectedCallback();
            }
            if (!this._attached) {
                this._attached = true;
                this._flushQueuedAttributes();
            }
        }
        disconnectedCallback() {
            if (super["disconnectedCallback"]) {
                super["disconnectedCallback"]();
            }
            if (this.i.disconnectedCallback) {
                this.i.disconnectedCallback();
            }
            if (this._attached) {
                this._attached = false;
            }
        }
        static get observedAttributes() {
            if (IgcToolActionComponent._observedAttributesIgcToolActionComponent == null) {
                let names = getAllPropertyNames(IgcToolActionComponent);
                for (let i = 0; i < names.length; i++) {
                    names[i] = toSpinal(names[i]);
                }
                IgcToolActionComponent._observedAttributesIgcToolActionComponent = names;
            }
            return IgcToolActionComponent._observedAttributesIgcToolActionComponent;
        }
        attributeChangedCallback(name, oldValue, newValue) {
            if (this._settingAttributes) {
                return;
            }
            let setName = fromSpinal(name);
            this._updatingFromAttribute = true;
            this[setName] = newValue;
            this._updatingFromAttribute = false;
        }
        get id() {
            return this.i.dk;
        }
        /**
         * Gets the sub actions associated with this tool action.
        */
        get actions() {
            if (this._actions === null) {
                let coll = new IgcToolActionCollection();
                let innerColl = this.i.actions;
                if (!innerColl) {
                    innerColl = new ToolActionCollection_internal();
                }
                this._actions = coll._fromInner(innerColl);
            }
            return this._actions;
        }
        set actions(v) {
            if (this._actions !== null) {
                this._actions._setSyncTarget(null);
                this._actions = null;
            }
            let coll = new IgcToolActionCollection();
            this._actions = coll._fromOuter(v);
            let syncColl = new SyncableObservableCollection$1(ToolAction.$type);
            let innerColl = this.i.actions;
            if (!innerColl) {
                innerColl = new ToolActionCollection_internal();
            }
            syncColl._inner = innerColl;
            syncColl.clear();
            this._actions._setSyncTarget(syncColl);
        }
        /**
         * Gets or sets the width to use for this tool action.
        */
        get width() {
            return this.i.b0;
        }
        set width(v) {
            this.i.b0 = +v;
            this._a("width", this.i.b0);
        }
        get parentId() {
            return this.i.dv;
        }
        set parentId(v) {
            this.i.dv = v;
        }
        /**
         * Gets or sets the Id of the action that this action will be inserted before.
         * Subsequent actions will continue to insert before the target action, following
         * the original action that set this property. If using this property do not use the
         * AfterId property.
         * This property is only relevant during initial insertion into the toolbar. Changing the property
         * after the action has already been added will have no effect.
        */
        get beforeId() {
            return this.i.cu;
        }
        set beforeId(v) {
            this.i.cu = v;
        }
        /**
         * Gets or sets the Id of the action that this action will be inserted after.
         * Subsequent actions will continue to insert after the target action, following
         * the original action that set this property. If using this property do not use the
         * BeforeId property.
         * This property is only relevant during initial insertion into the toolbar. Changing the property
         * after the action has already been added will have no effect.
        */
        get afterId() {
            return this.i.cr;
        }
        set afterId(v) {
            this.i.cr = v;
        }
        /**
         * Sets to the Id of the action that this action will overlay itself on to.
         * The purpose of this property is to allow users to override actions automatically generated by
         * toolbar targets such as the DataChart. These generated actions all have unique Ids associated
         * with them that can be used for the OverlayId.
         * These overlaid actions will
         * not
         * replace the generated one but will have its properties
         * overlaid onto the generated actions.
        */
        get overlayId() {
            return this.i.dq;
        }
        set overlayId(v) {
            this.i.dq = v;
        }
        get commandId() {
            return this.i.cz;
        }
        set commandId(v) {
            this.i.cz = v;
        }
        get commandArgument() {
            return this.i.cw;
        }
        set commandArgument(v) {
            this.i.cw = v;
        }
        get commandArgumentValue() {
            return this.i.b2;
        }
        set commandArgumentValue(v) {
            this.i.b2 = v;
        }
        get paddingLeft() {
            return this.i.bw;
        }
        set paddingLeft(v) {
            this.i.bw = +v;
            this._a("paddingLeft", this.i.bw);
        }
        get actualPaddingLeft() {
            return this.i.bh;
        }
        set actualPaddingLeft(v) {
            this.i.bh = +v;
            this._a("actualPaddingLeft", this.i.bh);
        }
        get paddingRight() {
            return this.i.bx;
        }
        set paddingRight(v) {
            this.i.bx = +v;
            this._a("paddingRight", this.i.bx);
        }
        get actualPaddingRight() {
            return this.i.bi;
        }
        set actualPaddingRight(v) {
            this.i.bi = +v;
            this._a("actualPaddingRight", this.i.bi);
        }
        get paddingBottom() {
            return this.i.bv;
        }
        set paddingBottom(v) {
            this.i.bv = +v;
            this._a("paddingBottom", this.i.bv);
        }
        get actualPaddingBottom() {
            return this.i.bg;
        }
        set actualPaddingBottom(v) {
            this.i.bg = +v;
            this._a("actualPaddingBottom", this.i.bg);
        }
        get paddingTop() {
            return this.i.by;
        }
        set paddingTop(v) {
            this.i.by = +v;
            this._a("paddingTop", this.i.by);
        }
        get actualPaddingTop() {
            return this.i.bj;
        }
        set actualPaddingTop(v) {
            this.i.bj = +v;
            this._a("actualPaddingTop", this.i.bj);
        }
        /**
         * Gets the actual hover background used by the tool action.
        */
        get actualHoverBackground() {
            return brushToString(this.i.ft);
        }
        set actualHoverBackground(v) {
            this.i.ft = stringToBrush(v);
            this._a("actualHoverBackground", brushToString(this.i.ft));
        }
        /**
         * Gets or sets the hover background for this tool action.
        */
        get hoverBackground() {
            return brushToString(this.i.f0);
        }
        set hoverBackground(v) {
            this.i.f0 = stringToBrush(v);
            this._a("hoverBackground", brushToString(this.i.f0));
        }
        /**
         * Gets the actual background used by the tool action.
        */
        get actualBackground() {
            return brushToString(this.i.fq);
        }
        set actualBackground(v) {
            this.i.fq = stringToBrush(v);
            this._a("actualBackground", brushToString(this.i.fq));
        }
        /**
         * Gets or sets the background color of the tool action.
        */
        get background() {
            return brushToString(this.i.fx);
        }
        set background(v) {
            this.i.fx = stringToBrush(v);
            this._a("background", brushToString(this.i.fx));
        }
        /**
         * Gets the actual label text color.
        */
        get actualTextColor() {
            return brushToString(this.i.fw);
        }
        set actualTextColor(v) {
            this.i.fw = stringToBrush(v);
            this._a("actualTextColor", brushToString(this.i.fw));
        }
        /**
         * Gets or sets the label text color.
        */
        get textColor() {
            return brushToString(this.i.f3);
        }
        set textColor(v) {
            this.i.f3 = stringToBrush(v);
            this._a("textColor", brushToString(this.i.f3));
        }
        /**
         * Gets the actual label text color when the tool is disabled.
        */
        get actualDisabledTextColor() {
            return brushToString(this.i.fr);
        }
        set actualDisabledTextColor(v) {
            this.i.fr = stringToBrush(v);
            this._a("actualDisabledTextColor", brushToString(this.i.fr));
        }
        /**
         * Gets or sets the label text color when the tool is disabled.
        */
        get disabledTextColor() {
            return brushToString(this.i.fy);
        }
        set disabledTextColor(v) {
            this.i.fy = stringToBrush(v);
            this._a("disabledTextColor", brushToString(this.i.fy));
        }
        /**
         * Gets the actual font used.
        */
        get actualTextStyle() {
            return this.i.u;
        }
        set actualTextStyle(v) {
            this.i.u = v;
        }
        /**
         * Gets or sets the font to use.
        */
        get textStyle() {
            if (this.i.w == null) {
                return null;
            }
            return this.i.w.fontString;
        }
        set textStyle(v) {
            let fi = new FontInfo();
            fi.fontString = v;
            this.i.w = fi;
            this._a("textStyle", this.i.w != null ? this.i.w.fontString : "");
        }
        get isDisabled() {
            return this.i.an;
        }
        set isDisabled(v) {
            this.i.an = ensureBool(v);
            this._a("isDisabled", this.i.an);
        }
        get isHighlighted() {
            return this.i.ao;
        }
        set isHighlighted(v) {
            this.i.ao = ensureBool(v);
            this._a("isHighlighted", this.i.ao);
        }
        get name() {
            return this.i.dn;
        }
        set name(v) {
            this.i.dn = v;
        }
        /**
         * Gets or sets the title of the tool.
        */
        get title() {
            return this.i.d4;
        }
        set title(v) {
            this.i.d4 = v;
        }
        /**
         * Gets or sets the subtitle of the tool.
        */
        get subtitle() {
            return this.i.d0;
        }
        set subtitle(v) {
            this.i.d0 = v;
        }
        get iconName() {
            return this.i.db;
        }
        set iconName(v) {
            this.i.db = v;
        }
        get iconCollectionName() {
            return this.i.c7;
        }
        set iconCollectionName(v) {
            this.i.c7 = v;
        }
        /**
         * Gets or sets the icon viewbox left.
        */
        get iconViewBoxLeft() {
            return this.i.bs;
        }
        set iconViewBoxLeft(v) {
            this.i.bs = +v;
            this._a("iconViewBoxLeft", this.i.bs);
        }
        /**
         * Gets or sets the icon viewbox top.
        */
        get iconViewBoxTop() {
            return this.i.bt;
        }
        set iconViewBoxTop(v) {
            this.i.bt = +v;
            this._a("iconViewBoxTop", this.i.bt);
        }
        /**
         * Gets or sets the icon viewbox width.
        */
        get iconViewBoxWidth() {
            return this.i.bu;
        }
        set iconViewBoxWidth(v) {
            this.i.bu = +v;
            this._a("iconViewBoxWidth", this.i.bu);
        }
        /**
         * Gets or sets the icon viewbox height.
        */
        get iconViewBoxHeight() {
            return this.i.br;
        }
        set iconViewBoxHeight(v) {
            this.i.br = +v;
            this._a("iconViewBoxHeight", this.i.br);
        }
        /**
         * Gets or sets the icon fill color.
        */
        get iconFill() {
            return brushToString(this.i.f1);
        }
        set iconFill(v) {
            this.i.f1 = stringToBrush(v);
            this._a("iconFill", brushToString(this.i.f1));
        }
        /**
         * Gets the actual icon fill color.
        */
        get actualIconFill() {
            return brushToString(this.i.fu);
        }
        set actualIconFill(v) {
            this.i.fu = stringToBrush(v);
            this._a("actualIconFill", brushToString(this.i.fu));
        }
        /**
         * Gets or sets the icon fill colors.
        */
        get iconFillColors() {
            return fromBrushCollection(this.i.c);
        }
        set iconFillColors(v) {
            this.i.c = toBrushCollection(v);
            this._a("iconFillColors", brushCollectionToString(this.i.c));
        }
        /**
         * Gets or sets the icon stroke color.
        */
        get iconStroke() {
            return brushToString(this.i.f2);
        }
        set iconStroke(v) {
            this.i.f2 = stringToBrush(v);
            this._a("iconStroke", brushToString(this.i.f2));
        }
        /**
         * Gets the actual icon stroke color.
        */
        get actualIconStroke() {
            return brushToString(this.i.fv);
        }
        set actualIconStroke(v) {
            this.i.fv = stringToBrush(v);
            this._a("actualIconStroke", brushToString(this.i.fv));
        }
        /**
         * Gets or sets the icon stroke colors.
        */
        get iconStrokeColors() {
            return fromBrushCollection(this.i.d);
        }
        set iconStrokeColors(v) {
            this.i.d = toBrushCollection(v);
            this._a("iconStrokeColors", brushCollectionToString(this.i.d));
        }
        /**
         * Gets or sets the icon viewbox width.
        */
        get iconStrokeWidth() {
            return this.i.bq;
        }
        set iconStrokeWidth(v) {
            this.i.bq = +v;
            this._a("iconStrokeWidth", this.i.bq);
        }
        /**
         * Gets or sets the tool action visibility.
        */
        get visibility() {
            return this.i.f5;
        }
        set visibility(v) {
            this.i.f5 = ensureEnum(Visibility_$type, v);
            this._a("visibility", enumToString(Visibility_$type, this.i.f5));
        }
        /**
         * Gets or sets if the tool action will close its parent dropdown when executed by the user.
        */
        get closeOnExecute() {
            return this.i.ai;
        }
        set closeOnExecute(v) {
            this.i.ai = ensureBool(v);
            this._a("closeOnExecute", this.i.ai);
        }
        get highlightWidth() {
            return this.i.bp;
        }
        set highlightWidth(v) {
            this.i.bp = +v;
            this._a("highlightWidth", this.i.bp);
        }
        get actualHighlightWidth() {
            return this.i.bf;
        }
        set actualHighlightWidth(v) {
            this.i.bf = +v;
            this._a("actualHighlightWidth", this.i.bf);
        }
        get highlightRadius() {
            return this.i.bo;
        }
        set highlightRadius(v) {
            this.i.bo = +v;
            this._a("highlightRadius", this.i.bo);
        }
        get actualHighlightRadius() {
            return this.i.be;
        }
        set actualHighlightRadius(v) {
            this.i.be = +v;
            this._a("actualHighlightRadius", this.i.be);
        }
        get highlightColor() {
            return brushToString(this.i.fz);
        }
        set highlightColor(v) {
            this.i.fz = stringToBrush(v);
            this._a("highlightColor", brushToString(this.i.fz));
        }
        get actualHighlightColor() {
            return brushToString(this.i.fs);
        }
        set actualHighlightColor(v) {
            this.i.fs = stringToBrush(v);
            this._a("actualHighlightColor", brushToString(this.i.fs));
        }
        findByName(name) {
            if (this.findEphemera) {
                if (name && name.indexOf("@@e:") == 0) {
                    return this.findEphemera(name);
                }
            }
            if (this.actions != null && this.actions.findByName && this.actions.findByName(name)) {
                return this.actions.findByName(name);
            }
            return null;
        }
        get hasUserValues() {
            return this._hasUserValues;
        }
        __m(propertyName) {
            if (!this._inStyling) {
                this._hasUserValues.add(propertyName);
            }
        }
        _styling(container, component, parent) {
            if (this._inStyling) {
                return;
            }
            this._inStyling = true;
            this._stylingContainer = container;
            this._stylingParent = component;
            let genericPrefix = "";
            let typeName = this.i.$type.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            genericPrefix = toSpinal("ToolActionComponent");
            let additionalPrefixes = [];
            let prefix = toSpinal(typeName);
            additionalPrefixes.push(prefix + "-");
            let b = this.i.$type.baseType;
            while (b && b.name != "Object" &&
                b.name != "Base" &&
                b.name != "Control" &&
                b.Name != "DependencyObject" &&
                b.Name != "FrameworkElement") {
                typeName = b.name;
                if (typeName.indexOf("Xam") === 0) {
                    typeName = typeName.substring(3);
                }
                let basePrefix = toSpinal(typeName);
                additionalPrefixes.push(basePrefix + "-");
                b = b.baseType;
            }
            if (parent) {
                let parentTypeName = parent.i.$type.name;
                if (parentTypeName.indexOf("Xam") === 0) {
                    parentTypeName = parentTypeName.substring(3);
                }
                let parentPrefix = toSpinal(parentTypeName);
                additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
                additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
            }
            initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
            if (this._otherStyling) {
                this._otherStyling(container, component, parent);
            }
            this._inStyling = false;
        }
        cleanUp() {
            this.i.d8();
        }
        fromInfo(info) {
            this.i.ej(info);
        }
        openSubMenu() {
            this.i.e0();
        }
        closeSubmenu() {
            this.i.ea();
        }
        get onCommand() {
            return this._onCommand;
        }
        set onCommand(ev) {
            if (this._onCommand_wrapped !== null) {
                this.i.onCommand = delegateRemove(this.i.onCommand, this._onCommand_wrapped);
                this._onCommand_wrapped = null;
                this._onCommand = null;
            }
            this._onCommand = ev;
            this._onCommand_wrapped = (o, e) => {
                let outerArgs = new IgcToolCommandEventArgs();
                outerArgs._provideImplementation(e);
                if (this.beforeOnCommand) {
                    this.beforeOnCommand(this, outerArgs);
                }
                if (this._onCommand) {
                    this._onCommand(this, outerArgs);
                }
            };
            this.i.onCommand = delegateCombine(this.i.onCommand, this._onCommand_wrapped);
            ;
        }
        get performed() {
            return this._performed;
        }
        set performed(ev) {
            if (this._performed_wrapped !== null) {
                this.i.performed = delegateRemove(this.i.performed, this._performed_wrapped);
                this._performed_wrapped = null;
                this._performed = null;
            }
            this._performed = ev;
            this._performed_wrapped = (o, e) => {
                let outerArgs = new IgcToolActionPerformedEventArgs();
                outerArgs._provideImplementation(e);
                if (this.beforePerformed) {
                    this.beforePerformed(this, outerArgs);
                }
                if (this._performed) {
                    this._performed(this, outerArgs);
                }
            };
            this.i.performed = delegateCombine(this.i.performed, this._performed_wrapped);
            ;
        }
    }
    IgcToolActionComponent._observedAttributesIgcToolActionComponent = null;
    return IgcToolActionComponent;
})();
