import { __extends, __values } from "tslib";
import { fromSize, ensureEnum, brushToString, stringToBrush, ensureBool, initializePropertiesFromCss, NamePatcher, toSpinal, enumToString, getAllPropertyNames, fromSpinal } from "igniteui-webcomponents-core";
import { FontInfo } from "igniteui-webcomponents-core";
import { WebComponentRenderer, PortalManager } from "igniteui-webcomponents-core";
import { delegateCombine, delegateRemove } from "igniteui-webcomponents-core";
import { CollectionAdapter } from "igniteui-webcomponents-core";
import { NotifyCollectionChangedAction } from "igniteui-webcomponents-core";
import { RegisterElementHelper } from "igniteui-webcomponents-core";
import { Toolbar } from "./Toolbar";
import { BaseControlTheme_$type } from "igniteui-webcomponents-core";
import { IgcHTMLElement } from "igniteui-webcomponents-core";
import { SyncableObservableCollection$1 } from "igniteui-webcomponents-core";
import { ToolbarOrientation_$type } from './ToolbarOrientation';
import { ToolActionCollection as ToolActionCollection_internal } from "./ToolActionCollection";
import { ToolAction } from "./ToolAction";
import { IgcToolActionComponent } from "./igc-tool-action-component";
import { IgcToolActionCollection } from "./igc-tool-action-collection";
import { IgcToolCommandEventArgs } from './igc-tool-command-event-args';
var IgcToolbarComponent = /** @class */ /*@__PURE__*/ (function (_super) {
    __extends(IgcToolbarComponent, _super);
    function IgcToolbarComponent() {
        var _this = _super.call(this) || this;
        _this._implementation = null;
        _this.contentActions = [];
        /**
        * The actions actually present in the editor. Do not directly modify this array.
        * This array's contents can be modified by using the actions property or providing child content.
        */
        _this.combinedActions = [];
        _this._actions = null;
        _this._actionsAdapter = null;
        _this._disconnected = false;
        _this._settingAttributes = false;
        _this._attached = false;
        _this._queuedSetAttributes = [];
        _this._updatingFromAttribute = false;
        _this._autoGeneratedActions = null;
        _this._actualActions = null;
        _this.__p = null;
        _this._hasUserValues = new Set();
        _this._stylingContainer = null;
        _this._stylingParent = null;
        _this._inStyling = false;
        _this._onCommand = null;
        _this._onCommand_wrapped = null;
        if (_this._styling) {
            NamePatcher.ensureStylablePatched(Object.getPrototypeOf(_this));
        }
        _this._implementation = _this.createImplementation();
        _this._implementation.externalObject = _this;
        _this._portalManager = new PortalManager("toolbarContent");
        _this._webComponentRenderer = new WebComponentRenderer(_this, document, true, {}, _this._portalManager);
        _this._webComponentWrapper = _this._webComponentRenderer.createElement("div");
        _this._webComponentRenderer.updateRoot(_this._webComponentWrapper);
        _this._webComponentWrapper.setStyleProperty("display", "block");
        _this._webComponentWrapper.setStyleProperty("width", "100%");
        _this._webComponentWrapper.setStyleProperty("height", "100%");
        _this._actionsAdapter = new CollectionAdapter(_this.contentActions, _this.i.actions, _this.combinedActions, function (c) { return c.i; }, function (i) {
            if (_this._webComponentRenderer && _this._webComponentRenderer.rootWrapper.getNativeElement().parentElement) {
                i._styling(_this, _this, _this);
            }
        }, function (i) { });
        _this.i.provideContainer(_this._webComponentRenderer);
        _this.i.notifySizeChanged();
        var mut = new MutationObserver(function (list) {
            var e_1, _b;
            try {
                for (var list_1 = __values(list), list_1_1 = list_1.next(); !list_1_1.done; list_1_1 = list_1.next()) {
                    var mutation = list_1_1.value;
                    if (mutation.type == 'childList') {
                        _this.updateContentProperties();
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (list_1_1 && !list_1_1.done && (_b = list_1.return))
                        _b.call(list_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        });
        mut.observe(_this, {
            childList: true
        });
        _this._webComponentRenderer.addSizeWatcher(function () {
            _this.i.notifySizeChanged();
        });
        return _this;
    }
    Object.defineProperty(IgcToolbarComponent.prototype, "height", {
        get: function () {
            return this._height;
        },
        set: function (value) {
            this._height = value;
            this.style.height = value;
            this.i.notifySizeChanged();
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "width", {
        get: function () {
            return this._width;
        },
        set: function (value) {
            this._width = value;
            this.style.width = value;
            this.i.notifySizeChanged();
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "actions", {
        /**
         * A collection of manually added actions for the editor.
        */
        get: function () {
            var _this = this;
            if (this._actions === null) {
                var coll = new IgcToolActionCollection();
                var inner = coll._innerColl;
                inner.addListener(function (sender, e) {
                    switch (e.action) {
                        case NotifyCollectionChangedAction.Add:
                            _this._actionsAdapter.insertManualItem(e.newStartingIndex, e.newItems.item(0));
                            break;
                        case NotifyCollectionChangedAction.Remove:
                            _this._actionsAdapter.removeManualItemAt(e.oldStartingIndex);
                            break;
                        case NotifyCollectionChangedAction.Replace:
                            _this._actionsAdapter.removeManualItemAt(e.oldStartingIndex);
                            _this._actionsAdapter.insertManualItem(e.newStartingIndex, e.newItems.item(0));
                            break;
                        case NotifyCollectionChangedAction.Reset:
                            _this._actionsAdapter.clearManualItems();
                            break;
                    }
                });
                this._actions = coll;
            }
            return this._actions;
        },
        enumerable: false,
        configurable: true
    });
    // supports angular themes or custom properties set in CSS
    IgcToolbarComponent.prototype.updateStyle = function () {
        this._styling(this, this);
    };
    IgcToolbarComponent.prototype.destroy = function () {
        this._implementation.destroy();
        this._webComponentRenderer.destroy();
    };
    IgcToolbarComponent.prototype.updateContentProperties = function () {
        this.contentActions.length = 0;
        for (var i = 0; i < this.children.length; i++) {
            if (this.children[i] instanceof IgcToolActionComponent) {
                this.contentActions.push(this.children[i]);
            }
        }
        if (this._actionsAdapter != null) {
            this._actionsAdapter.notifyContentChanged();
        }
    };
    IgcToolbarComponent.prototype.createImplementation = function () {
        return new Toolbar();
    };
    Object.defineProperty(IgcToolbarComponent.prototype, "i", {
        get: function () {
            return this._implementation;
        },
        enumerable: false,
        configurable: true
    });
    IgcToolbarComponent.prototype.disconnectedCallback = function () {
        this._disconnected = true;
    };
    IgcToolbarComponent.prototype.connectedCallback = function () {
        if (this._disconnected) {
            this._disconnected = false;
            return;
        }
        this.classList.add("ig-toolbar");
        this.classList.add("igc-toolbar");
        var rootWrapper = this._webComponentRenderer.rootWrapper;
        var rootElement = rootWrapper.getNativeElement();
        this.appendChild(rootElement);
        this._attached = true;
        this.style.display = "block";
        this.style.height = this._height;
        this.style.width = this._width;
        this._flushQueuedAttributes();
        // supports themes or custom properties set in CSS
        this._styling(this, this);
        for (var i = 0; i < this.combinedActions.length; i++) {
            var s = this.combinedActions[i];
            s._styling(this, this, this);
        }
        this.afterContentInit();
    };
    IgcToolbarComponent.prototype.afterContentInit = function () {
        this.i.notifySizeChanged();
    };
    Object.defineProperty(IgcToolbarComponent.prototype, "target", {
        /**
         * Gets or sets a target component which the toolbar can request actions from.
         */
        get: function () {
            var ret = this.i.target;
            if (!ret) {
                return ret;
            }
            if (ret.externalObject != null) {
                ret = ret.externalObject;
            }
            return ret;
        },
        set: function (v) {
            if (v && v._implementation) {
                v = v._implementation;
            }
            this.i.target = v;
        },
        enumerable: false,
        configurable: true
    });
    IgcToolbarComponent.prototype._enqueueSetAttribute = function (attrName, attrValue) {
        this._queuedSetAttributes.push({ attrName: attrName, attrValue: attrValue });
    };
    IgcToolbarComponent.prototype._flushQueuedAttributes = function () {
        this._settingAttributes = true;
        for (var i = 0; i < this._queuedSetAttributes.length; i++) {
            this.setAttribute(this._queuedSetAttributes[i].attrName, this._queuedSetAttributes[i].attrValue);
        }
        this._settingAttributes = false;
        this._queuedSetAttributes.length = 0;
    };
    IgcToolbarComponent.prototype._a = function (attrName, attrValue) {
        if (this._updatingFromAttribute) {
            return;
        }
        if (attrValue) {
            attrValue = attrValue.toString();
        }
        this._settingAttributes = true;
        attrName = toSpinal(attrName);
        if (this._attached) {
            this.setAttribute(attrName, attrValue);
        }
        else {
            this._enqueueSetAttribute(attrName, attrValue);
        }
        this._settingAttributes = false;
    };
    Object.defineProperty(IgcToolbarComponent, "observedAttributes", {
        get: function () {
            if (IgcToolbarComponent._observedAttributesIgcToolbarComponent == null) {
                var names = getAllPropertyNames(IgcToolbarComponent);
                for (var i = 0; i < names.length; i++) {
                    names[i] = toSpinal(names[i]);
                }
                IgcToolbarComponent._observedAttributesIgcToolbarComponent = names;
            }
            return IgcToolbarComponent._observedAttributesIgcToolbarComponent;
        },
        enumerable: false,
        configurable: true
    });
    IgcToolbarComponent.prototype.attributeChangedCallback = function (name, oldValue, newValue) {
        if (this._settingAttributes) {
            return;
        }
        var setName = fromSpinal(name);
        this._updatingFromAttribute = true;
        this[setName] = newValue;
        this._updatingFromAttribute = false;
    };
    IgcToolbarComponent.register = function () {
        if (!IgcToolbarComponent._isElementRegistered) {
            IgcToolbarComponent._isElementRegistered = true;
            RegisterElementHelper.registerElement(IgcToolbarComponent.htmlTagName, IgcToolbarComponent);
        }
    };
    Object.defineProperty(IgcToolbarComponent.prototype, "baseTheme", {
        /**
     * Gets or sets the base built in theme to use for the checkbox.
    */
        get: function () {
            return this.i.r;
        },
        set: function (v) {
            this.i.r = ensureEnum(BaseControlTheme_$type, v);
            this._a("baseTheme", enumToString(BaseControlTheme_$type, this.i.r));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "autoGeneratedActions", {
        /**
         * Gets or sets the data or data source instance to which to bind the grid.
        */
        get: function () {
            if (this._autoGeneratedActions === null) {
                var coll = new IgcToolActionCollection();
                var innerColl = this.i.autoGeneratedActions;
                if (!innerColl) {
                    innerColl = new ToolActionCollection_internal();
                }
                this._autoGeneratedActions = coll._fromInner(innerColl);
                this.i.autoGeneratedActions = innerColl;
            }
            return this._autoGeneratedActions;
        },
        set: function (v) {
            if (this._autoGeneratedActions !== null) {
                this._autoGeneratedActions._setSyncTarget(null);
                this._autoGeneratedActions = null;
            }
            var coll = new IgcToolActionCollection();
            this._autoGeneratedActions = coll._fromOuter(v);
            var syncColl = new SyncableObservableCollection$1(ToolAction.$type);
            var innerColl = this.i.autoGeneratedActions;
            if (!innerColl) {
                innerColl = new ToolActionCollection_internal();
            }
            syncColl._inner = innerColl;
            syncColl.clear();
            this._autoGeneratedActions._setSyncTarget(syncColl);
            this.i.autoGeneratedActions = innerColl;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "actualActions", {
        /**
         * Gets or sets the data or data source instance to which to bind the grid.
        */
        get: function () {
            if (this._actualActions === null) {
                var coll = new IgcToolActionCollection();
                var innerColl = this.i.actualActions;
                if (!innerColl) {
                    innerColl = new ToolActionCollection_internal();
                }
                this._actualActions = coll._fromInner(innerColl);
                this.i.actualActions = innerColl;
            }
            return this._actualActions;
        },
        set: function (v) {
            if (this._actualActions !== null) {
                this._actualActions._setSyncTarget(null);
                this._actualActions = null;
            }
            var coll = new IgcToolActionCollection();
            this._actualActions = coll._fromOuter(v);
            var syncColl = new SyncableObservableCollection$1(ToolAction.$type);
            var innerColl = this.i.actualActions;
            if (!innerColl) {
                innerColl = new ToolActionCollection_internal();
            }
            syncColl._inner = innerColl;
            syncColl.clear();
            this._actualActions._setSyncTarget(syncColl);
            this.i.actualActions = innerColl;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "orientation", {
        /**
         * Gets or sets the orientation of the toolbar.
        */
        get: function () {
            return this.i.l;
        },
        set: function (v) {
            this.i.l = ensureEnum(ToolbarOrientation_$type, v);
            this._a("orientation", enumToString(ToolbarOrientation_$type, this.i.l));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "backgroundColor", {
        /**
         * Gets or sets the color to use for the background of the component.
        */
        get: function () {
            return brushToString(this.i.de);
        },
        set: function (v) {
            this.i.de = stringToBrush(v);
            this._a("backgroundColor", brushToString(this.i.de));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "submenuBackgroundColor", {
        /**
         * Gets or sets the background color for submenus.
        */
        get: function () {
            return brushToString(this.i.dn);
        },
        set: function (v) {
            this.i.dn = stringToBrush(v);
            this._a("submenuBackgroundColor", brushToString(this.i.dn));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "groupHeaderBackgroundColor", {
        /**
         * Gets or sets the group header background color.
        */
        get: function () {
            return brushToString(this.i.dg);
        },
        set: function (v) {
            this.i.dg = stringToBrush(v);
            this._a("groupHeaderBackgroundColor", brushToString(this.i.dg));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "separatorBackgroundColor", {
        /**
         * Gets or sets the separator background color.
        */
        get: function () {
            return brushToString(this.i.dm);
        },
        set: function (v) {
            this.i.dm = stringToBrush(v);
            this._a("separatorBackgroundColor", brushToString(this.i.dm));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "hoverBackgroundColor", {
        /**
         * Gets or sets the hover background color.
        */
        get: function () {
            return brushToString(this.i.dj);
        },
        set: function (v) {
            this.i.dj = stringToBrush(v);
            this._a("hoverBackgroundColor", brushToString(this.i.dj));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "textColor", {
        /**
         * Gets or sets the color to use for the text of the component.
        */
        get: function () {
            return brushToString(this.i.dp);
        },
        set: function (v) {
            this.i.dp = stringToBrush(v);
            this._a("textColor", brushToString(this.i.dp));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "disabledTextColor", {
        /**
         * Gets or sets the color to use for the text in disabled tool actions.
        */
        get: function () {
            return brushToString(this.i.df);
        },
        set: function (v) {
            this.i.df = stringToBrush(v);
            this._a("disabledTextColor", brushToString(this.i.df));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "groupHeaderTextColor", {
        /**
         * Gets or sets the text color for group header tool actions.
        */
        get: function () {
            return brushToString(this.i.dh);
        },
        set: function (v) {
            this.i.dh = stringToBrush(v);
            this._a("groupHeaderTextColor", brushToString(this.i.dh));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "toolTipTextColor", {
        /**
         * Gets or sets the tooltip text color.
        */
        get: function () {
            return brushToString(this.i.dr);
        },
        set: function (v) {
            this.i.dr = stringToBrush(v);
            this._a("toolTipTextColor", brushToString(this.i.dr));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "toolTipBackgroundColor", {
        /**
         * Gets or sets the tooltip background color.
        */
        get: function () {
            return brushToString(this.i.dq);
        },
        set: function (v) {
            this.i.dq = stringToBrush(v);
            this._a("toolTipBackgroundColor", brushToString(this.i.dq));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "toolTipCornerRadius", {
        /**
         * Gets or sets the tooltip corner radius.
        */
        get: function () {
            return this.i.a3;
        },
        set: function (v) {
            this.i.a3 = +v;
            this._a("toolTipCornerRadius", this.i.a3);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "toolTipElevation", {
        /**
         * Gets or sets the shadow elevation for the tooltip.
        */
        get: function () {
            return this.i.a4;
        },
        set: function (v) {
            this.i.a4 = +v;
            this._a("toolTipElevation", this.i.a4);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "iconFill", {
        /**
         * Gets or sets the fill color of icons in the ToolPanel.
        */
        get: function () {
            return brushToString(this.i.dk);
        },
        set: function (v) {
            this.i.dk = stringToBrush(v);
            this._a("iconFill", brushToString(this.i.dk));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "iconStroke", {
        /**
         * Gets or sets the stroke color of icons in the ToolPanel.
        */
        get: function () {
            return brushToString(this.i.dl);
        },
        set: function (v) {
            this.i.dl = stringToBrush(v);
            this._a("iconStroke", brushToString(this.i.dl));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "cellTextStyle", {
        /**
         * Gets or Sets the property name that contains the values.
        */
        get: function () {
            if (this.i.u == null) {
                return null;
            }
            return this.i.u.fontString;
        },
        set: function (v) {
            var fi = new FontInfo();
            fi.fontString = v;
            this.i.u = fi;
            this._a("cellTextStyle", this.i.u != null ? this.i.u.fontString : "");
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "separatorVerticalPaddingLeft", {
        /**
         * Gets or sets the left padding for separators in the vertical orientation.
        */
        get: function () {
            return this.i.a0;
        },
        set: function (v) {
            this.i.a0 = +v;
            this._a("separatorVerticalPaddingLeft", this.i.a0);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "separatorVerticalPaddingRight", {
        /**
         * Gets or sets the right padding for separators in the vertical orientation.
        */
        get: function () {
            return this.i.a1;
        },
        set: function (v) {
            this.i.a1 = +v;
            this._a("separatorVerticalPaddingRight", this.i.a1);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "separatorVerticalPaddingTop", {
        /**
         * Gets or sets the top padding for separators in the vertical orientation.
        */
        get: function () {
            return this.i.a2;
        },
        set: function (v) {
            this.i.a2 = +v;
            this._a("separatorVerticalPaddingTop", this.i.a2);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "separatorVerticalPaddingBottom", {
        /**
         * Gets or sets the bottom padding for separators in the vertical orientation.
        */
        get: function () {
            return this.i.az;
        },
        set: function (v) {
            this.i.az = +v;
            this._a("separatorVerticalPaddingBottom", this.i.az);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "separatorHorizontalPaddingLeft", {
        /**
         * Gets or sets the left padding for separators in the horizontal orientation.
        */
        get: function () {
            return this.i.aw;
        },
        set: function (v) {
            this.i.aw = +v;
            this._a("separatorHorizontalPaddingLeft", this.i.aw);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "separatorHorizontalPaddingRight", {
        /**
         * Gets or sets the right padding for separators in the horizontal orientation.
        */
        get: function () {
            return this.i.ax;
        },
        set: function (v) {
            this.i.ax = +v;
            this._a("separatorHorizontalPaddingRight", this.i.ax);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "separatorHorizontalPaddingTop", {
        /**
         * Gets or sets the top padding for separators in the horizontal orientation.
        */
        get: function () {
            return this.i.ay;
        },
        set: function (v) {
            this.i.ay = +v;
            this._a("separatorHorizontalPaddingTop", this.i.ay);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "separatorHorizontalPaddingBottom", {
        /**
         * Gets or sets the bottom padding for separators in the horizontal orientation.
        */
        get: function () {
            return this.i.av;
        },
        set: function (v) {
            this.i.av = +v;
            this._a("separatorHorizontalPaddingBottom", this.i.av);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "showOnHover", {
        /**
         * Gets or sets if the tool actions should display their submenus on mouse hover.
        */
        get: function () {
            return this.i.ae;
        },
        set: function (v) {
            this.i.ae = ensureBool(v);
            this._a("showOnHover", this.i.ae);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "highlightWidth", {
        get: function () {
            return this.i.au;
        },
        set: function (v) {
            this.i.au = +v;
            this._a("highlightWidth", this.i.au);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "highlightRadius", {
        get: function () {
            return this.i.at;
        },
        set: function (v) {
            this.i.at = +v;
            this._a("highlightRadius", this.i.at);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "highlightColor", {
        get: function () {
            return brushToString(this.i.di);
        },
        set: function (v) {
            this.i.di = stringToBrush(v);
            this._a("highlightColor", brushToString(this.i.di));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolbarComponent.prototype, "rowHeight", {
        /**
         * Gets or sets the row height that should be used.
        */
        get: function () {
            return this.i.a6;
        },
        set: function (v) {
            this.i.a6 = +v;
            this._a("rowHeight", this.i.a6);
        },
        enumerable: false,
        configurable: true
    });
    IgcToolbarComponent.prototype.findByName = function (name) {
        if (this.findEphemera) {
            if (name && name.indexOf("@@e:") == 0) {
                return this.findEphemera(name);
            }
        }
        if (this.autoGeneratedActions != null && this.autoGeneratedActions.findByName && this.autoGeneratedActions.findByName(name)) {
            return this.autoGeneratedActions.findByName(name);
        }
        if (this.actions != null && this.actions.findByName && this.actions.findByName(name)) {
            return this.actions.findByName(name);
        }
        if (this.actualActions != null && this.actualActions.findByName && this.actualActions.findByName(name)) {
            return this.actualActions.findByName(name);
        }
        return null;
    };
    Object.defineProperty(IgcToolbarComponent.prototype, "hasUserValues", {
        get: function () {
            return this._hasUserValues;
        },
        enumerable: false,
        configurable: true
    });
    IgcToolbarComponent.prototype.__m = function (propertyName) {
        if (!this._inStyling) {
            this._hasUserValues.add(propertyName);
        }
    };
    IgcToolbarComponent.prototype._styling = function (container, component, parent) {
        if (this._inStyling) {
            return;
        }
        this._inStyling = true;
        this._stylingContainer = container;
        this._stylingParent = component;
        var genericPrefix = "";
        var typeName = this.i.$type.name;
        if (typeName.indexOf("Xam") === 0) {
            typeName = typeName.substring(3);
        }
        genericPrefix = toSpinal("ToolbarComponent");
        var additionalPrefixes = [];
        var prefix = toSpinal(typeName);
        additionalPrefixes.push(prefix + "-");
        var b = this.i.$type.baseType;
        while (b && b.name != "Object" &&
            b.name != "Base" &&
            b.name != "Control" &&
            b.Name != "DependencyObject" &&
            b.Name != "FrameworkElement") {
            typeName = b.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            var basePrefix = toSpinal(typeName);
            additionalPrefixes.push(basePrefix + "-");
            b = b.baseType;
        }
        if (parent) {
            var parentTypeName = parent.i.$type.name;
            if (parentTypeName.indexOf("Xam") === 0) {
                parentTypeName = parentTypeName.substring(3);
            }
            var parentPrefix = toSpinal(parentTypeName);
            additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
            additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
        }
        initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
        if (this._otherStyling) {
            this._otherStyling(container, component, parent);
        }
        this._inStyling = false;
    };
    /**
 * Get the desired size the toolbar wants to be.

*/
    IgcToolbarComponent.prototype.getDesiredSize = function () {
        var iv = this.i.ds();
        return fromSize(iv);
    };
    /**
     * Forces any open sub menus to close.
    
    */
    IgcToolbarComponent.prototype.closeSubmenus = function () {
        this.i.cg();
    };
    IgcToolbarComponent.prototype.onCommandStateChanged = function (id, type, value) {
        this.i.onCommandStateChanged(id, (type == null ? null : type), value);
    };
    IgcToolbarComponent.prototype.registerIconFromText = function (iconCollection, iconName, iconText) {
        this.i.cx(iconCollection, iconName, iconText);
    };
    IgcToolbarComponent.prototype.getIconFromCache = function (iconCollection, iconName) {
        var iv = this.i.bj(iconCollection, iconName);
        return (iv);
    };
    IgcToolbarComponent.prototype.registerMultiPathSVG = function (iconCollection, iconName, multiPathSVG) {
        this.i.cy(iconCollection, iconName, multiPathSVG);
    };
    IgcToolbarComponent.prototype.getMultiPathSVGFromCache = function (iconCollection, iconName) {
        var iv = this.i.a(iconCollection, iconName);
        return (iv);
    };
    Object.defineProperty(IgcToolbarComponent.prototype, "onCommand", {
        /**
         * Event raised when a command is triggered in the toolbar.
        */
        get: function () {
            return this._onCommand;
        },
        set: function (ev) {
            var _this = this;
            if (this._onCommand_wrapped !== null) {
                this.i.onCommand = delegateRemove(this.i.onCommand, this._onCommand_wrapped);
                this._onCommand_wrapped = null;
                this._onCommand = null;
            }
            this._onCommand = ev;
            this._onCommand_wrapped = function (o, e) {
                var outerArgs = new IgcToolCommandEventArgs();
                outerArgs._provideImplementation(e);
                if (_this.beforeOnCommand) {
                    _this.beforeOnCommand(_this, outerArgs);
                }
                if (_this._onCommand) {
                    _this._onCommand(_this, outerArgs);
                }
            };
            this.i.onCommand = delegateCombine(this.i.onCommand, this._onCommand_wrapped);
            ;
        },
        enumerable: false,
        configurable: true
    });
    IgcToolbarComponent._observedAttributesIgcToolbarComponent = null;
    IgcToolbarComponent.htmlTagName = "igc-toolbar";
    IgcToolbarComponent._isElementRegistered = false;
    return IgcToolbarComponent;
}(IgcHTMLElement));
export { IgcToolbarComponent };
